<?php

namespace App\Finance\Services\Api;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;

class BaiwangApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('baiwang_invoice');
        $this->getPlatformInfo();
    }

    public function setInvoicePlatform($platformName)
    {
        $this->invoicePlatform = $platformName;
    }

    /**
     * 查询企业唯一标识
     */
    public function queryQiyeUniqueSign($qiyeName)
    {
        try{
            if($this->checkPlatformStatus()){
                $qiyeParams = [
                    'nsrmc' => $this->platformInfo['platform_params'][$qiyeName]['nsrmc'],
                    'nsrsbh' => $this->platformInfo['platform_params'][$qiyeName]['nsrsbh']
                ];
                $response = $this->sendPost($this->getPlatformApiUrl().'/cloud/queryUniqueSign',$qiyeParams);
                return $this->response($response);
            }

        }catch(\Exception $exception){
            return $this->exception($exception);
        }
    }

    public function queryOrder($params)
    {
        try{
            if($this->checkPlatformStatus()){
                $queryParams = array_merge(
                    $params,
                    [
                        'qyId' => $this->platformInfo['platform_params'][$this->invoicePlatform]['qyid']
                    ]
                );
                $response = $this->sendPost($this->getPlatformApiUrl().'/cloud/queryOrder',$queryParams);
                return $this->response($response);
            }

        }catch(\Exception $exception){
            return $this->exception($exception);
        }
    }

    /**
     * 开票申请(蓝字&红字)
     * @param $params
     * @return mixed|void
     */
    public function applyElecInvoice($params)
   {
        try{
            if($this->checkPlatformStatus()){
                $requestParams = $params;
                $response = $this->sendPost($this->getPlatformApiUrl().'/cloud/applyElecInvoice',$requestParams);
                return $this->response($response);
            }

        }catch(\Throwable $exception){
            throw $exception;
            //return $this->exception($exception);
        }
    }

    /**
     * 蓝字发票作废（发票开具后开票日期在当月才能作废，跨月需要红冲。）
     * @param $params
     * @return mixed|void
     */
    public function cancelElecInvoice($params)
    {
        try{
            if($this->checkPlatformStatus()){
                $requestParams = $params;
                $response = $this->getPostClient($this->getPlatformApiUrl().'/cloud/elecInvoiceCancel',$requestParams);
                return $this->response($response);
            }

        }catch(\Exception $exception){
            return $this->exception($exception);
        }
    }

    /**
     * 红字发票撤销
     * @param $params
     * @return mixed|void
     */
    public function cancelElecRedInvoice($params)
    {
    }

    /**
     * 红字发票确认单状态查询
     * @param $params
     * @return mixed|void
     */
    public function queryRedInvoiceConfirmState($params)
    {
    }

    /**
     * 发票下载(支持单张发票pdf地址查询。)
     * @param $params
     * @return mixed|void
     */
    public function invoiceDownload($params)
    {
        try{
            if($this->checkPlatformStatus()){
                $requestParams = $params;
                $response = $this->getPostClient($this->getPlatformApiUrl().'/cloud/invoiceDownload',$requestParams);
                return $this->response($response);
            }

        }catch(\Exception $exception){
            return $this->exception($exception);
        }
    }

    private function exception($exception)
    {
        SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':'.__FUNCTION__.' exception => ');
        return false;
    }

    private function getPlatformApiUrl()
    {
        /*if(env('API_DEBUG')){
            return $this->platformInfo['platform_params']['test_url'];
        }*/
        return $this->platformInfo['platform_url'];
    }

    public function response($response)
    {
        $response = json_decode($response,true);
        if($response && $response['code'] == 0){
            return $response['data'];
        }elseif($response['code'] == -1){
            throw new \Exception($response['message']);
        }
        return false;
    }

    public function sendPost($uri, $paramsBody, $header = [])
    {
        $options = [];
        $options['headers'] = [
            //'Accept' => 'application/json;charset=UTF-8',
            'Content-Type'=> 'application/json;charset=UTF-8',
        ];

        $options['body'] = $this->requestParamsEncode($paramsBody);

        list($url,$method) = explode($this->platformInfo['platform_url'],$uri);
        // auth签名
        $header = array_merge($header,$this->authHeaders($method));

        if(!is_null($header)){
            $options['headers'] = array_merge($options['headers'],$header);
        }

        SimpleLogs::writeLog($paramsBody, __CLASS__.':'.$method.' 接口请求参数报文 => ');

        return $this->clientRequest('post', $uri, $options);
    }

    /**
     * 权限auth签名
     * @param $path
     * @return string[]
     */
    private function authHeaders($path)
    {
        $key = $this->platformInfo['platform_params'][$this->invoicePlatform]['key'];
        $sercret = $this->platformInfo['platform_params'][$this->invoicePlatform]['secret'];

        $serverName = "/bwdpzx/agentinvoiceservice-cloud/agentiscloud";
        $date = gmdate("D, d M Y H:i:s")." GMT";
        $host = "api.baiwangjs.com";
        $nonce = $this->uuid();
        $ua = "PHP ".PHP_VERSION." 64 bit, sdk-ver=1.0.0";

        $sign_str = "POST ".$serverName.$path." HTTP/1.1\n".
            "date: ".$date."\n".
            "host: ".$host."\n".
            "x-htjs-nonce: ".$nonce."\n".
            "x-htjs-ua: ".$ua;
        $signature = base64_encode(hash_hmac("sha256", $sign_str, $sercret, true));

        return [
            'date' => $date,
            'host' => $host,
            'x-htjs-nonce' => $nonce,
            'x-htjs-ua' => $ua,
            'Authorization' => 'hmac username="'.$key.'", algorithm="hmac-sha256", headers="request-line date host x-htjs-nonce x-htjs-ua", signature="'.$signature.'"'
        ];

    }

    public function base64UrlEncode($str)
    {
        return rtrim(strtr(base64_encode($str), "+/", "-_"), "=");
    }

    private function uuid()
    {
        $chars = md5(uniqid(mt_rand(), true));
        $uuid = substr ( $chars, 0, 8 ) . '-'
            . substr ( $chars, 8, 4 ) . '-'
            . substr ( $chars, 12, 4 ) . '-'
            . substr ( $chars, 16, 4 ) . '-'
            . substr ( $chars, 20, 12 );
        return $uuid ;
    }

    /**
     * 对参数进行base64加密
     * @param $params
     * @return string
     */
    public function requestParamsEncode($params)
    {
        $encodeParams = $params;
        if(is_array($encodeParams)){
            /*foreach($encodeParams as $key => $params){
                $encodeParams[$key] = base64_encode($params);
            }*/
            $encodeParams = base64_encode(json_encode($encodeParams));
            return $encodeParams;
        }
        return base64_encode($encodeParams);
    }

}
