<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1\Rhawn;

use App\Services\Api\InvoicesService;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

use OpenApi\Annotations\Post;
use OpenApi\Annotations\RequestBody;
use OpenApi\Annotations\MediaType;
use OpenApi\Annotations\Schema;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Response as AnnotationResponse;
use OpenApi\Annotations as OA;


class InvoiceController extends Controller
{
    use Helpers;

    public function __construct(InvoicesService $rhawnInvoicesService)
    {
        $this->rhawnInvoicesService= $rhawnInvoicesService;
        $this->controllerType = 'rhawn';
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/invoice/getSorderInvoiceItems",
     *     tags={"罗恩 - 发票相关接口"},
     *     summary="发票项详情数据",
     *     description="发票项详情数据",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"invoiceNumber"},
     *                 @Property(property="invoiceNumber", @Schema(type="string"),description="返回数据集数量"),
     *                 example={"invoiceNumber" : 0999899011}
     *             ),
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/InvoiceOrderItemsList")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getInvoiceSorderItems(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $orderItems = $this->rhawnInvoicesService->getInoviceItems($requestParams);
            if($orderItems){
                $orderItems = $this->formatKeysfromArray($orderItems,'toCamelCase');
            }
            return Response::success($orderItems,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取失败');
        }
    }


    /**
     * @Post(
     *     path="/openapi/rhawn/invoice/getSorderInvoiceList",
     *     tags={"罗恩 - 发票相关接口"},
     *     summary="订单发票列表数据",
     *     description="订单发票列表数据",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"orderNumber"},
     *                 @Property(property="orderNumber", @Schema(type="string"),description="返回数据集数量"),
     *                 example={"orderNumber" : 0999899011}
     *             ),
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/InvoiceOrderList")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getInvoiceListByOrderNumber(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $orderItems = $this->rhawnInvoicesService->getInoviceListByOrderNumber($requestParams);
            if($orderItems){
                $orderItems = $this->formatKeysfromArray($orderItems,'toCamelCase');
            }
            return Response::success($orderItems,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取失败');
        }
    }
}
