<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Http\Controllers\V1\Rhawn;

use App\Services\Api\RhawnOrdersService;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

use OpenApi\Annotations\Post;
use OpenApi\Annotations\RequestBody;
use OpenApi\Annotations\MediaType;
use OpenApi\Annotations\Schema;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Response as AnnotationResponse;
use OpenApi\Annotations as OA;


class OrdersController extends Controller
{
    use Helpers;

    public function __construct(RhawnOrdersService $rhawnOrdersService)
    {
       $this->rhawnOrdersService= $rhawnOrdersService;
       $this->controllerType = 'rhawn';
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/orders/getRhawnOrders",
     *     tags={"罗恩 - 订单相关接口"},
     *     summary="用户订单列表数据",
     *     description="订单列表",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"pageSize", "pageNo"},
     *                 @Property(property="pageSize", @Schema(type="integer"),description="返回数据集数量"),
     *                 @Property(property="pageNo", @Schema(type="integer"),description="当前页数"),
     *                 example={"pageSize" : 100, "pageNo" : 1}
     *             ),
     *         @OA\Examples(example="pageSize", value="100",summary="每页的数据量"),
     *         @OA\Examples(example="pageNo", value="1",summary="当前页数"),
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/OrdersList")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getCustomerOrders(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $orderList = $this->rhawnOrdersService->getCustomerRhawnOrdersList($requestParams);
            if($orderList){
                $orderList = $this->formatKeysfromArray($orderList,'toCamelCase');
            }
            return Response::success($orderList,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取订单列表失败');
        }
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/orders/getRhawnOrderDetail",
     *     tags={"罗恩 - 订单相关接口"},
     *     summary="用户订单详情数据",
     *     description="订单详情",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"orderNumber"},
     *                 @Property(property="orderNumber", @Schema(type="string"),description="订单编号"),
     *                 example={"orderNumber" : "20221008-122565111"}
     *             ),
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/OrdersDetail")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getCustomerOrderDetail(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $orderDetail = $this->rhawnOrdersService->getCustomerOrderDetail($requestParams);
            if($orderDetail){
                $orderDetail = $this->formatKeysfromArray($orderDetail,'toCamelCase');
            }
            return Response::success($orderDetail,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取订单详情失败');
        }
    }

    /**
     *
     * @Post(
     *     path="/openapi/rhawn/orders/createRhawnOrders",
     *     tags={"罗恩 - 订单相关接口"},
     *     summary="生成用户新订单",
     *     description="生成新订单",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"soCusPo", "soCaName","soCaMobile","soCaPhone","soCaProvince","soCaCity","soCaStreet","soCaCounty","soNote","items"},
     *                 @Property(property="soCusPo", @Schema(type="string"),description="客户订单编号"),
     *                 @Property(property="soCaName", @Schema(type="string"),description="收货人"),
     *                 @Property(property="soCaMobile", @Schema(type="string"),description="收货人手机"),
     *                 @Property(property="soCaPhone", @Schema(type="string"),description="收货人电话"),
     *                 @Property(property="soCaProvince", @Schema(type="string"),description="收货人省份"),
     *                 @Property(property="soCaCity", @Schema(type="string"),description="收货人城市"),
     *                 @Property(property="soCaStreet", @Schema(type="string"),description="收货人街道"),
     *                 @Property(property="soCaCounty", @Schema(type="string"),description="收货人区县"),
     *                 @Property(property="soNote", @Schema(type="string"),description="备注"),
     *                 @Property(property="items", type="array",
     *                      @OA\items(
     *                          @Property(property="pCode", @Schema(type="string"),description="产品code"),
     *                          @Property(property="num", @Schema(type="integer"),description="产品数量"),
     *                          @Property(property="siCusNote", @Schema(type="string"),description="备注"),
     *                          @Property(property="siSendNote", @Schema(type="string"),description="发货备注"),
     *                      ),description="当前页数"),
     *                 example={
                                "soCusPo": "2000000011111",
                                "soCaName": "张三",
                                "soCaMobile": "13321686555",
                                "soCaPhone": "61111111",
                                "soCaProvince": "上海",
                                "soCaCity": "上海市",
                                "soCaStreet": "望园路88888号888室",
                                "soNote": "测试",
                                "items": {{
                                            "pCode": "R000001-20mg",
                                            "num" : "1",
                                            "siCusNote" : "备注",
                                            "siSendNote" : "发货备注",
     *                                  }}
     *                         }
     *             )
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/OrdersDetail")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function createCustomerNewOrder(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $message = [
            'soCusPo.required' => "客户订单号必传",
            'soCaName.required' => "收货人必传",
            'soCaMobile.required' => "收货人手机必传",
            'soCaProvince.required' => "收货地省份必传",
            'soCaCity.required' => "收货地城市必传",
            'soCaStreet.required' => "收货地街道必传",
            'items.required' => "订单详情必传",
        ];
        $this->validateRequest($request, $message);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $orderDetail = $this->rhawnOrdersService->createNewCustomerOrders($requestParams);
            if($orderDetail){
                $orderDetail = $this->formatKeysfromArray($orderDetail,'toCamelCase');
            }
            return Response::success($orderDetail,'操作成功');
        }catch(\Throwable $exception){
            return Response::fail('生成订单失败. '. $exception->getMessage());
            //return $this->returnErrorExecptionResponse($exception,'生成订单失败');
        }
    }


    public function cancelCustomerOrder(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $result = $this->rhawnOrdersService->cancelCustomerOrder($requestParams);
            if($result){
                return Response::success([],'取消订单成功');
            }
            return Response::fail('取消订单失败',200);
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'取消订单失败');
        }
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/orders/getOrdersDispatch",
     *     tags={"罗恩 - 订单相关接口"},
     *     summary="订单的配货单信息",
     *     description="配货单信息",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"orderNumber"},
     *                 @Property(property="orderNumber", @Schema(type="string"),description="订单编号"),
     *                 example={"orderNumber" : "20221008-122565111"}
     *             ),
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/OrdersDispatchList")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getOrdersDispatch(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $dpDetail = $this->rhawnOrdersService->getOrderDispatchByOrderNumber($requestParams);
            if($dpDetail){
                $dpDetail = $this->formatKeysfromArray($dpDetail,'toCamelCase');
            }
            return Response::success($dpDetail,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取订单配货单信息失败');
        }
    }
}
