<?php

namespace App\Http\Controllers\V1\Rhawn;

use App\Services\Api\RhawnOrdersService;
use App\Services\Api\RhawnProductsService;
use Illuminate\Http\Request;
use Jiannei\Response\Laravel\Support\Facades\Response;
use App\Http\Controllers\V1\Controller;
use App\Support\Traits\Helpers;

use OpenApi\Annotations\Post;
use OpenApi\Annotations\RequestBody;
use OpenApi\Annotations\MediaType;
use OpenApi\Annotations\Schema;
use OpenApi\Annotations\Property;
use OpenApi\Annotations\Response as AnnotationResponse;
use OpenApi\Annotations as OA;

class ProductsController extends Controller
{
    use Helpers;

    public function __construct(RhawnProductsService $rhawnProductsService)
    {
       $this->rhawnProductsService = $rhawnProductsService;
       $this->controllerType = 'rhawn';
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/products/getAllProducts",
     *     tags={"罗恩 - 产品相关接口"},
     *     summary="获取所有产品列表数据集",
     *     description="产品列表",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"pageSize", "pageNo"},
     *                 @Property(property="pageSize", @Schema(type="integer"),description="返回数据集数量"),
     *                 @Property(property="pageNo", @Schema(type="integer"),description="当前页数"),
     *                 example={"pageSize" : 100, "pageNo" : 1}
     *             )
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/ProductsList")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getAllProductsList(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);

        try{
            $productList = $this->rhawnProductsService->getAllProducts($requestParams);
            if($productList){
                $productList = $this->formatKeysfromArray($productList,'toCamelCase');
            }
            return Response::success($productList,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取产品列表失败');
        }
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/products/getProductDetail",
     *     tags={"罗恩 - 产品相关接口"},
     *     summary="获取产品详情数据集",
     *     description="产品详情",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"productCode"},
     *                 @Property(property="productCode", @Schema(type="string"),description="产品编号"),
     *                 example={"productCode" : "R003250-1ml"}
     *             )
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/ProductDetailProperty")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getProductDetail(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $productDetail = $this->rhawnProductsService->getProductDetail($requestParams);
            if($productDetail){
                $productDetail = $this->formatKeysfromArray($productDetail,'toCamelCase');
            }
            return Response::success($productDetail,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取产品详情失败');
        }
    }

    /**
     * @Post(
     *     path="/openapi/rhawn/products/getProductByCas",
     *     tags={"罗恩 - 产品相关接口"},
     *     summary="根据cas号获取产品详情数据集",
     *     description="产品详情",
     *     @RequestBody(
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 required={"cas"},
     *                 @Property(property="cas", @Schema(type="string"),description="cas"),
     *                 example={"cas" : "57960-19-7"}
     *             )
     *         )
     *     ),
     *     @OA\Parameter(
     *         description="用户获取的token值",
     *         in="header",
     *         name="authorization",
     *         required=true,
     *         @OA\Schema(type="string"),
     *         @OA\Examples(example="authorization", value="bearerNWJiNDhkNzlmNjg0N2FlMmZiYjliZWM3NGVkNzIyMjNleUpsZUhCcGNtVWlPakUyTmpRMk1EazJORGNzSW1oaGMyZ2lPaUl5ZEhsc1JIQlhkWFpNUVdaWGJVRllJbjA9",summary=""),
     *     ),
     *     @AnnotationResponse(
     *         response="200",
     *         description="正常操作响应",
     *         @MediaType(
     *             mediaType="application/json",
     *             @Schema(
     *                 allOf={
     *                     @Schema(ref="#/components/schemas/ApiResponse"),
     *                     @Schema(
     *                         type="object",
     *                         @Property(property="data", ref="#/components/schemas/ProductByCas")
     *                     )
     *                 }
     *             )
     *         )
     *     ),
     *     security={
     *         {"bearer_token":{}}
     *     }
     * )
     */
    public function getProductByCas(Request $request)
    {
        $customerCode = $request->get('customer_code');
        if(!$customerCode){
            return Response::ok('客户编号不存在!');
        }
        $this->checkCustomerType($customerCode,$this->controllerType);

        $requestParams = $this->formatKeysfromArray($request->all(),'toUnderScore');
        $requestParams = array_merge($requestParams,['customer_code' => $customerCode]);
        try{
            $productDetail = $this->rhawnProductsService->getProductByCas($requestParams);
            if($productDetail){
                $productDetail = $this->formatKeysfromArray($productDetail,'toCamelCase');
            }
            return Response::success($productDetail,'操作成功');
        }catch(\Throwable $exception){
            return $this->returnErrorExecptionResponse($exception,'获取产品详情失败');
        }
    }

}
