<?php

namespace App\Providers\Auth\Jwt;

use Dingo\Api\Auth\Provider\Authorization;
use Exception;
use Dingo\Api\Routing\Route;
use Illuminate\Http\Request;
use Symfony\Component\HttpKernel\Exception\UnauthorizedHttpException;

use App\Repositories\Enums\ResponseCodeEnum;
use App\Support\Traits\Helpers ;
use Tymon\JWTAuth\JWTAuth;

class AdminAuthProvider extends Authorization
{
    use Helpers;


    /**
     * The JWTAuth instance.
     *
     * @var \Tymon\JWTAuth\JWTAuth
     */
    protected $auth;


    public function __construct(JWTAuth $auth){
       $this->auth = $auth;
    }

    /**
     * Authenticate request with a JWT.
     *
     * @param \Illuminate\Http\Request $request
     * @param \Dingo\Api\Routing\Route $route
     * @return mixed
     */
    public function authenticate(Request $request, Route $route)
    {
        try{
            $token = $this->getToken($request);
            $decodeToken = $this->decodeToken($token);
            if(empty($decodeToken) || (!is_array($decodeToken) && !$decodeToken['hash'])){
                throw new UnauthorizedHttpException('JWTAuth', 'Unable to authenticate with invalid token.');
            }
            if (! $user = $this->auth->setToken($decodeToken['hash'])->authenticate()) {
                throw new UnauthorizedHttpException('JWTAuth', 'Unable to authenticate with invalid token.');
            }
        }catch(Exception $exception){
            if($exception instanceof UnauthorizedHttpException){
                throw $exception;
            }
            throw new UnauthorizedHttpException('AdminAuth', $exception->getMessage(),null, ResponseCodeEnum::CLIENT_PARAMETER_ERROR);
        }

        return $user;
    }

    /**
     * Get the JWT from the request.
     *
     * @param \Illuminate\Http\Request $request
     * @return string
     *
     * @throws \Exception
     */
    protected function getToken(Request $request)
    {
        try {
            $this->validateAuthorizationHeader($request);

            $token = $this->parseAuthorizationHeader($request);
        } catch (Exception $exception) {
            if (! $token = $request->query('token', false)) {
                throw $exception;
            }
        }

        return $token;
    }

    /**
     * Parse JWT from the authorization header.
     *
     * @param \Illuminate\Http\Request $request
     * @return string
     */
    protected function parseAuthorizationHeader(Request $request)
    {
        return trim(str_ireplace($this->getAuthorizationMethod(), '', $request->header('authorization')));
    }

    /**
     * Get the providers authorization method.
     *
     * @return string
     */
    public function getAuthorizationMethod()
    {
        return 'bearer';
    }
}
