<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\ThirdPlatform\Api;

use App\ImportExport\WuxiLab\Export\WuxiLabExport;
use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Facades\SimpleLogs;
use App\Support\Traits\HttpClientHelpers;


class WuxiApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('wuxi_appTec');
        $this->getPlatformInfo();
    }

    /**
     * 批量更新商品信息
     * @param $product
     * @return bool
     */
    public function pushBatchUpdateProduct($product)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['chemicalProducts'] = $this->mergeApiMetaData($product);

                $response = $this->getPostClient($this->platformInfo['platform_url'].'/api/v1/update',$params, [
                    $this->platformInfo['platform_params']['username'], $this->platformInfo['platform_params']['password']
                ]);

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    public function backgroundLogin()
    {
        if($this->checkPlatformStatus()){
            $response = $this->getPostClient($this->platformInfo['platform_params']['adminUrl'].'/json_login',[
                'username' => $this->platformInfo['platform_params']['username'],
                'password' => $this->platformInfo['platform_params']['password']
            ],null,[
                'cookie' => 'org.springframework.web.servlet.i18n.CookieLocaleResolver.LOCALE=zh_CN; JSESSIONID=805B829F0A377E7777326FBBA5F859E8'
            ]);

            return json_decode($response,true)['token'];
        }
    }


    public function pushBatchUploadFile($type, $fileName, $token)
    {
        if($this->checkPlatformStatus()){

            switch($type){
                case 'product' : $type = 'PRODUCT_CHEMICAL';break;
                case 'package' : $type = 'PACKAGE_CHEMICAL';break;
                case 'stock' : $type = 'INVENTORY_CHEMICAL';break;
            }

            $params = [];
            $params['companyId'] = $this->platformInfo['platform_params']['companyId'];
            $params['type'] = $type;
            $params['batchId'] = '65d30cab-92b7-8d4a-ae00-d5474c93d4f7';

            $filePath = app(WuxiLabExport::class)->getSavePath($fileName);
            $filePath = storage_path('app/'.$filePath);
            $params['file']['name'] = $fileName.'.xlsx';
            $params['file']['contents'] = \GuzzleHttp\Psr7\Utils::tryFopen($filePath.'.xlsx','r');

            try{
                $response = $this->getMultipartPostClient($this->platformInfo['platform_params']['adminUrl'].'/excel/chemical/upload',$params,null,[
                    'cookie' => $this->getCookie($token),
                ]);
                if($this->apiResponse($response)){
                    unset($params['file']);
                    $urlParams = [];
                    foreach($params as $key => $param){
                        $urlParams[] = $key.'='.$param;
                    }
                    $confirmResponse = $this->clientRequest('post',
                        $this->platformInfo['platform_params']['adminUrl'].'/excel/finish?'.implode('&',$urlParams),
                        [
                            'headers' => [
                                'cookie' => $this->getCookie($token),
                                'content-type' => 'application/json;charset=UTF-8'
                            ]
                        ]);
                    $result = $this->apiResponse($confirmResponse);
                    if($result['success'] == true){
                       return true;
                    }
                }
                return false;
            }catch(\Throwable $exception){
                SimpleLogs::writeLog($exception->getMessage(), __CLASS__.':pushBatchUploadProducts', 'error');
                //file_put_contents('./error.txt',$exception->getMessage());
            }
        }
    }

    /**
     * @更新包装规格
     * @param $packages
     * @param $token
     * @return false|mixed|void
     */
    public function pushBatchUpdatePackages($packages,$token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['company_id'] = $this->platformInfo['platform_params']['companyId'];
                $params['product_type'] = 'CHAMICAL';
                $params = array_merge($params,$packages);

                $response = $this->getPostClient($this->platformInfo['platform_params']['adminUrl'].'/package/specification/save',$params,null, [
                    'cookie' => $this->getCookie($token)
                ]);

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    public function getPackageList($token)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['companyId'] = $this->platformInfo['platform_params']['companyId'];
                $params['productType'] = 'CHAMICAL';

                $urlParams = [];
                foreach($params as $key => $param){
                    $urlParams[] = $key.'='.$param;
                }

                $response = $this->clientRequest('get',
                    $this->platformInfo['platform_params']['adminUrl'].'/package/specification/count?'.implode('&',$urlParams),
                    [
                        'headers' => [
                            'cookie' => $this->getCookie($token),
                            'content-type' => 'application/json;charset=UTF-8'
                        ]
                    ]);
                if($response && $response > 0){
                    $page = 1;
                    $count = ceil($response / 30);

                    $packageList = [];

                    while(true){
                        $params['page'] = $page;
                        $params['rows'] = 30;
                        $urlParams = [];
                        foreach($params as $key => $param){
                            $urlParams[] = $key.'='.$param;
                        }
                        $packageListResponse = $this->clientRequest('get',
                            $this->platformInfo['platform_params']['adminUrl'].'/package/specification/list?'.implode('&',$urlParams),
                            [
                                'headers' => [
                                    'cookie' => $this->getCookie($token),
                                    'content-type' => 'application/json;charset=UTF-8'
                                ]
                            ]);

                        $result = $this->apiResponse($packageListResponse);
                        if($result){
                            $packageList = array_merge($packageList,$result);
                        }

                        $page ++;
                        if($page > $count){
                            break;
                        }
                        sleep(2);
                    }

                    return $packageList;
                }

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            $this->requestError($e);
        }
    }

    private function getCookie($token)
    {
        return 'org.springframework.web.servlet.i18n.CookieLocaleResolver.LOCALE=zh_CN;Auth-Token='.$token.';';
    }

    /**
     * 整理组装 api 数据
     * @param $metaData
     * @return array
     */
    public function mergeApiMetaData($metaData)
    {
        if(empty($metaData)){
           return [];
        }

        $mergeData = [];
        foreach($metaData['rawList'] as $datas){
            $tempData['supplier_product_id'] = $datas['r_code'];
            $tempData['mdl'] = '';
            $tempData['cas'] = $datas['c_cas'];
            $tempData['nu'] = '';
            $tempData['chinese_name'] = $datas['r_cn_name'];
            $tempData['english_name'] = $datas['r_en_name'];
            $tempData['chemicalformula'] = '';
            $tempData['purity'] = $datas['r_level'];
            /*$tempData['usually_ships_days_min'] = '';
            $tempData['usually_ships_within_days'] = '';
            $tempData['backorder_lead_time'] = '';*/
            $tempData['status'] = $datas['r_status'] == '1' ? 'ACTIVE' : 'INACTIVE';

            $tempData['inventorys'][] = [
                'type' => $datas['r_unit'],
                'quantity' => floatval($datas['r_stock']),
                'country' => 'CN',
                'stock_status' => $datas['r_stock'] > 0 ? 'INSTOCK' : 'OUTSTOCK',
            ];

            $tempData['packages'] = [];

            if(isset($metaData['packageList'][$datas['r_code']])){
                $package = $metaData['packageList'][$datas['r_code']];

                $pack['unit_description'] = $package['p_pack_unit'];
                $pack['package_status'] = $package['p_status'] == '1' && $package['p_show'] == '1' && $package['p_if_big'] == 0 ? 'ACTIVE' : 'INACTIVE';
                $tempData['status'] = $package['p_status'] == '1' && $package['p_show'] == '1' && $package['p_if_big'] == 0 ? 'ACTIVE' : 'INACTIVE';

                $pack['inventorys'] = [
                    'package_quantity' => $package['p_stock'],
                    'stock_status' => $package['p_stock'] > 0 ? 'INSTOCK' : 'OUTSTOK',
                    'country' => 'CN'
                ];

                $pack['prices'] = [
                    'price' => $package['p_price'],
                    'currency' => 'RMB',
                    'status' => 'ACTIVE'
                ];

                $pack['costs'] = [
                    'In_cost' => $package['p_avg_cost'],
                    'In_cost_currency' => 'RMB',
                    'status' => $package['p_stock'] > 0 ? 'ACTIVE' : 'INACTVIE',
                ];

                $pack['wuxiPackagePricings'] = [
                    'discount'=> 0.00,
                    'currency' => 'RMB',
                    'status' => 'INACTIVE'
                ];

                array_push($tempData['packages'],$pack);
            }

            array_push($mergeData,$tempData);
            unset($tempData);
        }

        return $mergeData;
    }

    protected function apiResponse($response)
    {
        $decodeResponse = json_decode($response,true);
        if($decodeResponse){
            return $decodeResponse;
        }
        return false;
    }

    protected function requestError($response)
    {
        SimpleLogs::writeLog($response->getMessage(), 'WuxiApi error', 'error');
        throw $response;
    }
}
